#ifndef __EXT2Obj_h__
#define __EXT2Obj_h__

#include "fs/FSResult.H"
#include "fs/FSObjImpl.H"
#include "fs/ext2/EXT2FS.H"

/**
 * As abstraction of EXT2Dir and EXT2File the main purpose of EXT2Obj is
 * to handle references of objects of the filesystem. 
 */
class EXT2Obj : public FSObjImpl
{
  /** number of the current references */
  int reference_count;
  /** number of the starting inode of the filesystem object */
  unsigned long	inode;
protected:
  /** pointer to the interface of the filesystem */
  EXT2FS *fs;
public:
  /**
   * Default constructor of the ext2-filesystem.
   *
   * @param filesystem pointer to the current filesystem
   * @param inodenr number of starting inode of the filesystem object
   */
  EXT2Obj(EXT2FS *filesystem, unsigned long inodenr);
  virtual ~EXT2Obj() {}
 
  /** 
   * Get the pointer to the interface of the filesystem.
   *
   * @return Pointer to the filesystem interface.
   */
  EXT2FS *filesystem ();
  
  /**
   * Adds a new reference to the filesystem object by increasing the reference 
   * counter.  
   */
  void new_reference ();

  /**
   * Removes a reference from the filesystem object by decreasing the reference 
   * counter. 
   */
  void release_reference ();
};

#endif /* __EXT2Obj_h__ */
