#ifndef __SNFSObj_h__
#define __SNFSObj_h__

#include "fs/FSResult.H"
#include "fs/FSObjImpl.H"
#include "fs/snfs/SNFSFS.H"

/**
 * As abstraction of SNFSDir and SNFSFile the main purpose of SNFSObj is
 * to handle references of objects of the filesystem. 
 */
class SNFSObj : public FSObjImpl
{
  /** number of the current references */
  int reference_count;
  
  /** filehandle for the filesystem object */
  unsigned long	handle;//ToDo: Typ des filehandles
protected:
  /** pointer to the interface of the filesystem */
  SNFSFS *fs;
public:
  /**
   * Default constructor of the SNFS-filesystem.
   *
   * @param filesystem pointer to the current filesystem
   * @param handlenr filehandle for the filesystem object
   */
  SNFSObj(SNFSFS *filesystem, unsigned long handlenr);
  virtual ~SNFSObj() {}
 
  /** 
   * Get the pointer to the interface of the filesystem.
   *
   * @return Pointer to the filesystem interface.
   */
  SNFSFS *filesystem ();
  
  /**
   * Adds a new reference to the filesystem object by increasing the reference 
   * counter.  
   */
  void new_reference ();

  /**
   * Removes a reference from the filesystem object by decreasing the reference 
   * counter. 
   */
  void release_reference ();
};

#endif /* __SNFSObj_h__ */
