/*
 * The maximum number of bytes of data in a READ or WRITE request.
 */
const MAXDATA = 8192;

/* The maximum number of bytes in a file name argument. */
const MAXNAMLEN = 255;

/* The size in bytes of the opaque "cookie" passed by READDIR. */
const COOKIESIZE  = 4;

/* The size in bytes of the opaque file handle. */
const FHSIZE = 32;

typedef opaque fhandle[FHSIZE];
typedef opaque nfsdata[MAXDATA];
typedef string filename<MAXNAMLEN>;
typedef string path<MAXPATHLEN>;

enum ftype {
  NFNON = 0,
  NFREG = 1,
  NFDIR = 2,
  NFBLK = 3,
  NFCHR = 4,
  NFLNK = 5
};

enum stat {
  NFS_OK             = 0,
  NFSERR_PERM        = 1,
  NFSERR_NOENT       = 2,
  NFSERR_IO          = 5,
  NFSERR_NXIO        = 6,
  NFSERR_ACCES       = 13,
  NFSERR_EXIST       = 17,
  NFSERR_NODEV       = 19,
  NFSERR_NOTDIR      = 20,
  NFSERR_ISDIR       = 21,
  NFSERR_FBIG        = 27,
  NFSERR_NOSPC       = 28,
  NFSERR_ROFS        = 30,
  NFSERR_NAMETOOLONG = 63,
  NFSERR_NOTEMPTY    = 66,
  NFSERR_DQUOT       = 69,
  NFSERR_STALE       = 70,
  NFSERR_WFLUSH      = 99
};

struct timevale {
  unsigned int seconds;
  unsigned int useconds;
};

struct sattr {
  unsigned int mode;
  unsigned int uid;
  unsigned int gid;
  unsigned int size;
  timevale      atime;
  timevale      mtime;
};

struct diropargs {
  fhandle  dir;
  filename name;
};

struct createargs {
  diropargs where;
  sattr attributes;
};

struct entry {
  unsigned fileid;
  filename name;
  entry *nextentry;
};

struct fattr {
  ftype        type;
  unsigned int mode;
  unsigned int nlink;
  unsigned int uid;
  unsigned int gid;
  unsigned int size;
  unsigned int blocksize;
  unsigned int rdev;
  unsigned int blocks;
  unsigned int fsid;
  unsigned int fileid;
  timevale      atime;
  timevale      mtime;
  timevale      ctime;
};

struct linkargs {
  fhandle from;
  diropargs to;
};

struct readargs {
  fhandle file;
  unsigned offset;
  unsigned count;
  unsigned totalcount;
};

struct readdirargs {
  fhandle dir;
  unsigned count;
};

struct renameargs {
  diropargs from;
  diropargs to;
};

struct sattrargs {
  fhandle file;
  sattr attributes;
};

struct writeargs {
  fhandle file;
  unsigned beginoffset;
  unsigned offset;
  unsigned totalcount;
  nfsdata data;
};

/*
 *
 */
union attrstat switch (stat status) {
  case NFS_OK:
    fattr attributes;
  default:
    void;
};

struct diropok_t {
  fhandle file;
  fattr attributes;
};

union diropres switch (stat status) {
  case NFS_OK:
    diropok_t diropok;
  default:
    void;
};

struct readdirok_t {
  entry *entries;
  bool eof;
};

union readdirres switch (stat status) {
  case NFS_OK:
    readdirok_t readdirok;
  default:
    void;
};

struct readresdata_t {
  fattr attributes;
  nfsdata data;
};

union readres switch (stat status) {
  case NFS_OK:
    readresdata_t readresdata;
  default:
    void;
};

struct statfsres_t {
  unsigned tsize;
  unsigned bsize;
  unsigned blocks;
  unsigned bfree;
  unsigned bavail;
};

union statfsres switch (stat status) {
  case NFS_OK:
    statfsres_t info;
  default:
    void;
};

/*
 * Remote file service routines
 */
program NFS_PROGRAM {
  version NFS_VERSION {
    void
    NFSPROC_NULL(void)              = 0;
    attrstat
    NFSPROC_GETATTR(fhandle)        = 1;
    attrstat
    NFSPROC_SETATTR(sattrargs)      = 2;
    diropres
    NFSPROC_LOOKUP(diropargs)       = 3;
    void
    NFSPROC_ACCESS(void)            = 4;
    readres
    NFSPROC_READ(readargs)          = 5;
    attrstat
    NFSPROC_WRITE(writeargs)        = 6;
    diropres
    NFSPROC_CREATE(createargs)      = 7;
    stat
    NFSPROC_REMOVE(diropargs)       = 8;
    stat
    NFSPROC_RENAME(renameargs)      = 9;
    stat
    NFSPROC_LINK(linkargs)          = 10;
    diropres
    NFSPROC_MKDIR(createargs)       = 11;
    stat
    NFSPROC_RMDIR(diropargs)        = 12;
    readdirres
    NFSPROC_READDIR(readdirargs)    = 13;
    diropres
    NFSPROC_ROOT(void)              = 14;
    statfsres
    NFSPROC_FSSTAT(fhandle)         = 15;
  } = 1;
} = 20000000;
