#include "fs/snfs/SNFSFS.H"
#include "fs/snfs/SNFSDir.H"
#include "fs/snfs/errno.h"
#include "fs/attributs.h"

#include <iostream>

//------------------------------------------------------------------------------
extern "C"
{
  FSResult str2perm(const char *val, unsigned short &perm);
  FSResult perm2str(unsigned short perm, char *val, int &size);
  FSResult type2str(unsigned short mode, char *&val, int &size);
}

static struct attrib attribs[] = 
{
  { "readonly"  , "Is the readonly mode set ? (\"yes\"/\"no\")"},
  { "uid"     , "UID for file access (number)" }, 
  { "gid"     , "GID for for file access (number)" }, 
  { "umask"   , "Unix style UMASK for file access (number)" }, 
  { "type"    , "filetype for creating new files (\"symlink\"/\"file\")" } 
};

SNFSFS::SNFSFS(char *host) 
{
  errno = 0;
  root_dir = NULL;
  server = strdup(host);

  if (!server) {
    // ToDo: check if server is reachable
    errno = SNFSE_FS_NULL_DEVICE;
    return;
  }
  
  // ToDo: get filehandle for rootdir -> create object for directory
  if (!root_dir) {
    errno = SNFSE_FS_ROOT_FAILED;
    return;
  }
}

SNFSFS::~SNFSFS()
{
  //ToDo: delete root directory
}


FSResult SNFSFS::check()
{
  switch (errno) 
  {
    case 0:                                                 // everything's fine
      return FSResult::NoError;
    case SNFSE_BLKDEV_INIT_FAILED|SNFSE_BLKDEV_READ_FAILED: // server failed
      return FSResult::FsSpecificError;
    case SNFSE_FS_NULL_DEVICE:                              // NULL pointer as server
      return FSResult::UninitializedObj;
    case SNFSE_FS_INIT_FAILED|SNFSE_FS_ROOT_FAILED:
      return FSResult::UninitializedObj;
    case SNFSE_SUPERBLK_BADMAGICNUMBER:
      return FSResult::InvalidFS;
    default:                                                // hmm... dunno
      return FSResult::FsSpecificError;
  }
}

FSResult SNFSFS::sync()
{
  if (errno) return check();
  //ToTo: synchronise with server
  int result = SNFSE_NOERROR;
  switch (result)
  {
    case SNFSE_NOERROR:
      return FSResult::NoError;
    case SNFSE_READONLY:
      return FSResult::Readonly;     
    default: 
      return FSResult::FsSpecificError;
  }
}

FSDirImpl* SNFSFS::root()
{
  return root_dir;
}

FSResult SNFSFS::attribute_count(int &count)
{
  count = NUMBER_OF_ATTRIBS;
  return FSResult::NoError;
}

FSResult SNFSFS::attribute_name(unsigned int n, const char *&name)
{
  if (n > NUMBER_OF_ATTRIBS) return FSResult::AttribUnknown;
  name = attribs[n].name;
  return FSResult::NoError;
}

FSResult SNFSFS::attribute_description(unsigned int n, const char *&description)
{
  if (n > NUMBER_OF_ATTRIBS) return FSResult::AttribUnknown;
  description = attribs[n].desc;
  return FSResult::NoError;
}

FSResult SNFSFS::get_attribute (const char* name, char *val, int& size)
{ 
  for (int i = 0; i < 5;i++)
  {
    if (strcmp(name,attribs[i].name)==0)
    {
      switch (i) 
      {
        case 0: // readonly
          //ToDo: insert source code
          break; 
        case 1: // uid
          //ToDo: insert source code
          break; 
        case 2: // gid
          //ToDo: insert source code
          break; 
        case 3: // umask
          //ToDo: insert source code
          break; 
        case 4: // type
          //ToDo: insert source code
          break;
      }
      if (size<0) return FSResult::BufferTooSmall;
      else return FSResult::NoError; 
    }
  }
  return FSResult::AttribUnknown;
}

FSResult SNFSFS::set_attribute(const char* name, const char *val)
{
  for (int i = 0; i < 5;i++)
  {
    if (strcmp(name,attribs[i].name)==0)
    {
      switch (i) 
      {
        case 0: // readonly
          //ToDo: insert source code
          break; 
        case 1: // uid
          //ToDo: insert source code
          break; 
        case 2: // gid
          //ToDo: insert source code
          break; 
        case 3: // umask
          //ToDo: insert source code
          break; 
        case 4: // type
          //ToDo: insert source code
          break;
      }
      return FSResult::NoError;
    }
  }
  return FSResult::AttribUnknown;
}
